import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  select,
} from 'd3';

export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin;
  let radius;
  let xAxisLabel;
  let yAxisLabel;

  const chart = (selection) => {
    const xScale = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const yScale = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    selection.each(function () {
      const svg = select(this)
        .append('svg')
        .attr('width', width)
        .attr('height', height);

      const g = svg
        .append('g')
        .attr(
          'transform',
          `translate(${margin.left}, ${margin.top})`,
        );

      // Draw circles
      g.selectAll('circle')
        .data(data)
        .enter()
        .append('circle')
        .attr('cx', (d) => xScale(xValue(d)))
        .attr('cy', (d) => yScale(yValue(d)))
        .attr('r', radius)
        .attr('fill', 'steelblue');

      // Draw x-axis
      g.append('g')
        .attr(
          'transform',
          `translate(0, ${height - margin.bottom})`,
        )
        .call(axisBottom(xScale))
        .append('text')
        .attr('x', width / 2)
        .attr('y', margin.bottom - 10)
        .attr('fill', '#000')
        .attr('text-anchor', 'middle')
        .text(xAxisLabel);

      // Draw y-axis
      g.append('g')
        .attr('transform', `translate(${margin.left}, 0)`)
        .call(axisLeft(yScale))
        .append('text')
        .attr('transform', 'rotate(-90)')
        .attr('y', 6)
        .attr('dy', '-5.5em')
        .attr('text-anchor', 'middle')
        .attr('fill', '#000')
        .text(yAxisLabel);
    });
  };

  chart.width = function (_) {
    if (!arguments.length) return width;
    width = _;
    return chart;
  };

  chart.height = function (_) {
    if (!arguments.length) return height;
    height = _;
    return chart;
  };

  chart.data = function (_) {
    if (!arguments.length) return data;
    data = _;
    return chart;
  };

  chart.xValue = function (_) {
    if (!arguments.length) return xValue;
    xValue = _;
    return chart;
  };

  chart.yValue = function (_) {
    if (!arguments.length) return yValue;
    yValue = _;
    return chart;
  };

  chart.margin = function (_) {
    if (!arguments.length) return margin;
    margin = _;
    return chart;
  };

  chart.radius = function (_) {
    if (!arguments.length) return radius;
    radius = _;
    return chart;
  };

  chart.xAxisLabel = function (_) {
    if (!arguments.length) return xAxisLabel;
    xAxisLabel = _;
    return chart;
  };

  chart.yAxisLabel = function (_) {
    if (!arguments.length) return yAxisLabel;
    yAxisLabel = _;
    return chart;
  };

  return chart;
};
